# -*- coding: utf-8 -*-
"""
Created on Wed Jun 14 18:16:45 2023

@author: peruzzetto
"""

import tilupy.raster
import tilupy.read

import os
import argparse
import glob
    

def process_results(fn_name, model, res_name, folder=None, param_files=None,
                    kwargs_read=None, **kwargs_fn):
    
    assert(model is not None)
    
    if folder is None:
        folder = os.getcwd()
        
    if param_files is None:
        param_files = '*.txt'
        
    print(folder, param_files)
        
    param_files = glob.glob(param_files, root_dir=folder)
    
    if len(param_files) == 0:
        print('No parameter file matching param_files pattern was found')
        return
        
    if kwargs_read is None:
        kwargs_read = dict()
     
    kw_read = dict(folder_base=folder)   
    kw_read.update(kwargs_read)
     
    for param_file in param_files:
        print_str = 'Processing simulation {:s}, {:s} {:s} .....'
        print(print_str.format(param_file, fn_name, res_name))
        kw_read['file_params'] = param_file
        res = tilupy.read.get_results(model, **kw_read)
        getattr(res, fn_name)(res_name, **kwargs_fn)
        
def to_raster(model=None, res_name='h', param_files=None, folder=None, 
              kwargs_read=None, **kwargs):
    
    kw = dict(fmt='asc')
    kw.update(kwargs)
    
    process_results('save', model, res_name,
                    folder=folder, param_files=param_files,
                    kwargs_read=kwargs_read, **kw)
    
def plot_results(model=None, res_name='h', param_files=None, folder=None, 
                 kwargs_read=None, **kwargs):
    
    kw = dict(save=True)
    kw.update(kwargs)
    
    process_results('plot', model, res_name,
                    folder=folder, param_files=param_files,
                    kwargs_read=kwargs_read, **kwargs)
    
def _get_parser(prog, description):
    parser = argparse.ArgumentParser(prog=prog,
                                     description=description,
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('model', help="Model name",
                        type=str)
    parser.add_argument('-n', '--res_name', help="Name of output, only for maps",
                        default='h', type=str,
                        )
    parser.add_argument('-p', '--param_files', help="Parameter file (globbing)",
                        default='*.txt', type=str)
    parser.add_argument('-f', '--folder', help="Root folder, default is current folder",
                        default=None, type=str)
    return parser

def _tilupy_plot():
    parser = _get_parser('tilupy_plot', 'Plot thin-layer simulation results')
    parser.add_argument('--fmt', help=("Plot output format "
                                       "(any accepted by matplotlib.savefig)"),
                        default='png', type=str,
                        )
    parser.add_argument('--vmin', help=("Minimum plotted value, "
                                        "adapted to data by default"),
                        default=None, type=float,
                        )
    parser.add_argument('--vmax', help=("Maximum plotted value, "
                                        "adapted to data by default"),
                        default=None, type=float,
                        )
    parser.add_argument('--minval_abs', help=("Minimum plotted absolute value,"
                                              " adapted to data by default"),
                        default=None, type=float,
                        )
    args = parser.parse_args()
    plot_results(**vars(args))
    
def _tilupy_to_raster():
    parser = _get_parser('tilupy_to_raster',
                         'Convert simulation results to rasters')
    parser.add_argument('--fmt', help=("File output format, "
                                       "tif/tiff requires rasterio"),
                        default='asc', type=str,
                        choices=['tif', 'tiff', 'txt', 'asc', 'ascii'])
    args = parser.parse_args()
    # plot_results(parser.model, parser.res_name)
    to_raster(**vars(args))
    
    
if __name__ == '__main__':
    
    # folder = 'd:/Documents/peruzzetto/tmp/test_shaltop/7p30e04_m3/coulomb'
    # plot_results('shaltop', 'h_max', '*18p00.txt', folder=folder)
    _tilupy_plot()
    


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Jun  1 15:27:36 2021

@author: peruzzetto
"""
import os

import numpy as np
import tilupy.read

# Dictionnary with results names lookup table, to match code output names
LOOKUP_NAMES = dict(h='rho', hvert='rho',
                    u='unorm', ux='u', uy='ut',
                    hu='momentum', hu2int='ek',
                    vol='vol')

# Classify results
STATES_OUTPUT = ['h', 'ux', 'uy', 'hvert']
forces = ['facc', 'fcurv', 'ffric', 'fgrav', 'fpression']
FORCES_OUTPUT = []
for axis in ['x', 'y']:
    for f in forces:
        FORCES_OUTPUT.append(f + axis)
        
INTEGRATED_OUTPUT = ['ek', 'ep', 'etot']


def read_params(file):
    """
    Read simulation parameters from file.

    Parameters
    ----------
    file : str
        Parameters file.

    Returns
    -------
    params : dict
        Dictionnary with parameters

    """
    if file is None:
        params = None
    else:
        params = dict()
        with open(file, 'r') as f:
            for line in f:
                (key, val) = line.split(' ')
                try:
                    params[key] = float(val)
                except ValueError:
                    params[key] = val.rstrip()
        params['nx'] = int(params['nx'])
        params['ny'] = int(params['ny'])

    return params

def get_axes(**varargs):
    """Set x and y axes."""
    if 'x0' in varargs :
        x0 = varargs['x0']
    else:
        x0 = 0
    if 'y0' in varargs:
        y0 = varargs['y0']
    else:
        y0 = 0

    nx = varargs['nx']
    ny = varargs['ny']
    dx = varargs['per']/nx
    dy = varargs['pery']/ny
    x = dx*np.arange(nx)+dx/2
    y = dy*np.arange(ny)+dy/2

    try:
        coord_pos = varargs['coord_pos']
    except KeyError:
        coord_pos = 'bottom_left'

    if coord_pos == 'bottom_left':
        x = x+x0
        y = y+y0
    elif coord_pos == 'upper_right':
        x = x+x0-x[-1]
        y = y+y0-y[-1]
    elif coord_pos == 'upper_left':
        x = x+x0
        y = y+y0-y[-1]
    elif coord_pos == 'lower_right':
        x = x+x0-x[-1]
        y = y+y0

    return x, y

def read_file_bin(file, nx, ny):
    """Read shaltop .bin result file."""
    data = np.fromfile(file, dtype=np.float32)
    nbim = int(np.size(data)/nx/ny)
    data = np.reshape(data, (nx, ny, nbim), order='F')
    data = np.transpose(np.flip(data, axis=1), (1, 0, 2))

    return data


def read_file_init(file, nx, ny):
    """Read shaltop initial .d files."""
    data = np.loadtxt(file)
    data = np.reshape(data, (nx, ny), order='F')
    data = np.transpose(np.flip(data, axis=1), (1, 0))
    return data


class Results(tilupy.read.Results):
    """Results of shaltop simulations."""

    def __init__(self, file_params=None, folder_base=None, **varargs):
        """
        Init simulation results.

        Parameters
        ----------
        file_params : str
            File where simulation parameters will be read

        """
        super().__init__()
        
        if folder_base is None:
            folder_base = os.getcwd()
        if file_params is None:
            file_params = 'params.txt'
            
        if '.' not in file_params:
            file_params = file_params + '.txt'
            
        file_params = os.path.join(folder_base, file_params)
        
        params = read_params(file_params)
        x, y = get_axes(**params)
        
        varargs.update(dict(code='shaltop',
                            htype='normal',
                            params=params,
                            x=x, y=y,
                            nx=len(x), ny=len(y)))
        
        for key in varargs:
            setattr(self, key, varargs[key])

        self.folder_base = folder_base
        # Folder where results are stored
        if 'folder_output' not in self.params:
            self.folder_output = os.path.join(self.folder_base,
                                              'data2')
        else:
            self.folder_output = os.path.join(self.folder_base,
                                              self.params['folder_output'])

        # Get time of outputs
        self.tim = np.loadtxt(os.path.join(self.folder_output, 'time_im.d'))

    
        
    @property
    def zinit(self):
        """ Compute or get cos(slope) of topography """
        if self._zinit is None:
            self.set_zinit()
        return self._zinit

    def set_zinit(self, zinit=None):
        """Set zinit, initial topography."""
        path_zinit = os.path.join(self.folder_base, self.folder_output,
                                  'z.bin')
        if not os.path.isfile(path_zinit) and 'file_z_init' in self.params:
            path_zinit = os.path.join(self.folder_base,
                                      self.params['file_z_init'])
            self._zinit = read_file_init(path_zinit, self.nx, self.ny)  
        else:
            self._zinit = np.squeeze(read_file_bin(path_zinit, self.nx, self.ny))

    def get_temporal_output(self, name, d=None, t=None, **varargs):
        """
        Read 2D time dependent simulation results.

        Parameters
        ----------
        name : str
            Name of output.
        d : ndarray, optional
            Data to be read. If None, will be computed.
            The default is None.
        t : ndarray, optional
            Time of results snapshots (1D array), matching last dimension of d.
            If None, will be computed. The default is None.
        **varargs : TYPE
            DESCRIPTION.

        """
        # Read thicknesses or velocity components
        if name in STATES_OUTPUT:
            file = os.path.join(self.folder_output,
                                LOOKUP_NAMES[name] + '.bin')
            d = read_file_bin(file, self.nx, self.ny)
            if name == 'hvert':
                d = d/self.costh[:, :, np.newaxis]
            t = self.tim
            
        # Raed integrated kinetic energy
        if name in ['hu2int']:
            array = np.loadtxt(os.path.join(self.folder_output,
                                            LOOKUP_NAMES[name] + '.d'))
            d = array[:, 1]
            t = array[:, 0]

        # Compute the velocity
        if name == 'u':
            d = self.get_u()
            t = self.tim

        if name in ['hu', 'hu2']:
            fileh = os.path.join(self.folder_output,
                                 'rho.bin')
            h = read_file_bin(fileh, self.nx, self.ny)
            u = self.get_u()
            if name == 'hu':
                d = h*u
            elif name == 'hu2':
                d = h*u**2
            t = self.tim
            
        if 'h_thresh' in varargs and varargs['h_thresh'] is not None and d.ndim==3:
            d = tilupy.read.use_thickness_threshold(self, d,
                                                  varargs['h_thresh'])

        return tilupy.read.TemporalResults(name, d, t)

    def get_static_output(self, name,
                          d=None, from_file=True, **varargs):
        """
        Read 2D time dependent simulation results.

        Parameters
        ----------
        name : str
            Name of output.
        d : ndarray, optional
            Data to be read. Last dimension is for time.
            If None, will be computed.
            The default is None.
        **varargs : TYPE
            DESCRIPTION.

        """
        if name in tilupy.read.COMPUTED_STATIC_DATA_2D:
            state, stat = name.split('_')
            if stat in ['final', 'initial']:
                hh = self.get_temporal_output(state)
                if state == 'hvert':
                    hh.d = hh.d/self.costh[:, :, np.newaxis]
                return hh.get_temporal_stat(stat)
            if from_file:
                file = os.path.join(self.folder_output,
                                    LOOKUP_NAMES[state] + stat + '.bin')
                if os.path.isfile(file):
                    d = np.squeeze(read_file_bin(file, self.nx, self.ny))
                else:
                    print(file + ' was not found, ' + name + '_' + stat
                          + ' computed from temporal output.')
                    from_file = False
            if not from_file:
                data = self.get_temporal_output(state)
                if state == 'hvert':
                    hh.d = hh.d/self.costh
                d = data.get_temporal_stat(stat).d
        else:
             raise(NotImplementedError())   

        return tilupy.read.StaticResults(name, d)

    def get_u(self):
        """ Compute velocity norm from results """
        file = os.path.join(self.folder_output,
                            'u' + '.bin')
        u = read_file_bin(file, self.nx, self.ny)
        file = os.path.join(self.folder_output,
                            'ut' + '.bin')
        ut = read_file_bin(file, self.nx, self.ny)

        [Fx, Fy] = np.gradient(self.zinit, self.y, self.x)
        u = u*self.costh[:, :, np.newaxis]
        ut = ut*self.costh[:, :, np.newaxis]
        d = np.sqrt(u**2 + ut**2 + (Fx[:, :, np.newaxis]*u
                                    + Fy[:, :, np.newaxis]*ut)**2)

        return d



_default_config = dict(language='english')

class Config(dict):
    
    def __init__(self, **kwargs):
        for key in kwargs:
            self[key] = kwargs[key]
            
    def update(self, **kwargs):
        for key in kwargs:
            self[key] = kwargs[key]
            
config = Config(**_default_config)

def set_config(**kwargs):
    config.update(**kwargs)
    
    if 'language' in kwargs:
        import tilupy.notations
        tilupy.notations.set_labels()
    
#import tilupy.notations
    


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Jun  3 12:29:52 2021

@author: peruzzetto
"""

import numpy as np
import importlib

def read_raster(file):
    if file.endswith('.asc') or file.endswith('.txt'):
        return read_ascii(file)
    elif file.endswith('.tif') or file.endswith('.tif'):
        return read_tiff(file)

def read_tiff(file):
    
    import rasterio
    with rasterio.open(file, 'r') as src:
        dem = src.read(1)
        ny, nx = dem.shape
        x = np.linspace(src.bounds.left, src.bounds.right, nx)
        y = np.linspace(src.bounds.bottom, src.bounds.top, ny)
    return x, y, dem
    
def read_ascii(file):
    """
    Read ascii grid file to numpy ndarray.

    Parameters
    ----------
    file : TYPE
        DESCRIPTION.

    Returns
    -------
    None.

    """
    dem = np.loadtxt(file, skiprows=6)
    grid = {}
    with open(file, 'r') as fid:
        for i in range(6):
            tmp = fid.readline().split()
            grid[tmp[0]] = float(tmp[1])
    try:
        x0 = grid['xllcenter']
        y0 = grid['yllcenter']
    except KeyError:
        x0 = grid['xllcorner']
        y0 = grid['yllcorner']
    nx = int(grid['ncols'])
    ny = int(grid['nrows'])
    dx = dy = grid['cellsize']
    x = np.linspace(x0, x0+(nx-1)*dx, nx)
    y = np.linspace(y0, y0+(ny-1)*dy, ny)

    return x, y, dem

def write_tiff(x, y, z, file_out, **kwargs):
    """
    Write raster as tif file

    Parameters
    ----------
    x : TYPE
        DESCRIPTION.
    y : TYPE
        DESCRIPTION.
    z : TYPE
        DESCRIPTION.
    file_out : TYPE
        DESCRIPTION.

    Returns
    -------
    None.

    """
    import rasterio
    from rasterio.transform import Affine
    if 'driver' not in kwargs:
        kwargs['driver'] = 'GTiff'
    res = (x[-1]-x[0])/(len(x)-1)
    transform = Affine.translation(x[0] - res / 2, y[-1] - res / 2)\
        * Affine.scale(res, -res)
    with rasterio.open(
            file_out,
            'w',
            height=z.shape[0],
            width=z.shape[1],
            count=1,
            dtype=z.dtype,
            transform=transform,
            **kwargs) as dst:
        dst.write(z, 1)
    

def write_ascii(x, y, z, file_out):
    """
    Write raster as ascii file.

    Parameters
    ----------
    x : TYPE
        DESCRIPTION.
    y : TYPE
        DESCRIPTION.
    z : TYPE
        DESCRIPTION.

    Returns
    -------
    None.

    """
    nx = z.shape[1]
    ny = z.shape[0]
    cellsize = x[1] - x[0]
    header_txt = 'ncols {:.0f}\nnrows {:.0f}\nxllcorner {:.5f}\nyllcorner {:.5f}\n'
    header_txt += 'cellsize {:.4f}\nnodata_value -99999'
    header_txt = header_txt.format(nx, ny, x[0], y[0], cellsize)
    np.savetxt(file_out, z, header=header_txt, comments='')
    
def write_raster(x, y, z, file_out, fmt=None, **kwargs):
    
    # File format read from file_out overrides fmt
    fmt = file_out.split('.')
    if len(fmt)>1:
        fmt = fmt[-1]
    else:
        file_out = file_out + '.' + fmt
    
    if fmt not in ['asc', 'ascii', 'txt', 'tif', 'tiff']:
        raise ValueError('File format not implemented in write_raster')
        
    if fmt.startswith('tif'):
        if importlib.util.find_spec('rasterio') is None:
            print(('rasterio is required to write tif files.',
                   ' Switching to asc format'))
            fmt = 'asc'
        
    if fmt in ['asc', 'ascii', 'txt']:
        write_ascii(x, y, z, file_out)
    elif fmt in ['tif', 'tiff']:
        write_tiff(x, y, z, file_out, **kwargs)
    else:
        raise NotImplementedError()
